############################################################################
##########################  PRELIMINARY STUFF  #############################
############################################################################

# This script relies heavily on the package adegenet
# It was written with v. 2.0.1
# However the package changes some of its base functions with updates
# Future versions of adegenet might require future versions of this script.

packages <- c("introgress", "HIest", "adegenet", "pegas", "akima", 
              "poppr", "mmod", "strataG", "plyr", "ggplot2", "reshape2", 
              "gridExtra","ggfortify","egg","grid","gtable","ggtern")
lapply(packages, require, character.only = TRUE)

# Define function: opposite of "%in%
`%ni%` <- Negate(`%in%`)

# Set working directory
setwd("~/Desktop/ContactZones") # Working directory

# Source plotting function for sPCA
source("./scripts/sPCAplot.R")


# Set parameters for the script. Choose these carefully.

percent <- c(0.01)   
# Prop. of top ranked sPCA1 or FST loci to use
# this can be a vector of values

a.not.NA <- 0.5            
# Proportion of individuals 
# without missing data in all individuals

p.invar <- T               
# Remove loci that are invariant in P1/P2

max.freq <- 0.95           
# Maximum frequency of most frequent allele
# in order to keep that locus (e.g. 0.9)
# Implemented this way for loci with >2 alleles 
# (e.g. A = 0.85, B = 0.10, C = 0.05)

het.cutoff <- 0.25         
# Global Ho cutoff to throw out false loci
# loci > 0.5 will be automatically discarded
# between het.cutoff and 0.5, HWE will be checked
# if Ho > He significantly, locus is thrown out

sPC.num <- c(1,2,3)        
# Which spatial PC column to use as hybrid zone?
# this can be a vector of PCs
# For this dataset, the first 3 correspond to...
# 1 ~ East-North, 2 ~ East-South, 3 ~ East-West
# Choose hybrid zone(s) even if not doing sPCA

snpORhaplo <- "haplo"        # Use SNP ("snp") or haplotype ("haplo") dataset
if(snpORhaplo == "snp") { n.loc <- 2468 }   # Set number of loci
if(snpORhaplo == "haplo") { n.loc <- 2468 } # Set number of loci

Input.Process.Data = T     
# Input and process genetics dataset?
# Takes awhile. If already done, may want to skip.

Perform.sPCA = T      
# Perform sPCA? ** ONLY CHOOSE THIS OR FST **
# Takes awhile. If already done, may want to skip.



############################################################################
##########################  INPUT/PROCESS DATA #############################
############################################################################

# Genotype file in structure format
if(Input.Process.Data==T) {
  # SNP or haplotypes file, depending upon choice above
  gi <- read.structure(paste0("./data/YOSE", snpORhaplo, ".stru"),
                       n.ind = 535, n.loc = n.loc, onerowperind = F,
                       col.lab = 1, col.pop = 2, row.marknames = 1,
                       NA.char = "NA", ask = F, quiet=T)
  
  # Remove monomorphic loci (across all 3 zones) for sPCA
  print(paste0("Removed ", length(which(gi@loc.n.all == 1)), " monomorphic loci."))
  remove1 <- names(which(gi@loc.n.all == 1))
  gi <- gi[loc=which(gi@loc.n.all > 1), drop=T]
  
  # Remove loci with excess heterozygosity (see beginning) for sPCA
  sum <- summary(gi)
  print(paste0("Removed ", length(which(sum$Ho >= 0.5)), " loci with Ho >= 0.5."))
  gi <- gi[loc=which(sum$Ho < 0.5), drop=T]
  sum <- summary(gi)
  if(sum(sum$Ho > het.cutoff) > 0) {
    gi.temp <- gi[loc=which(sum$Ho > het.cutoff), drop=T]
    hwe <- data.frame(hw.test(gi.temp, B=1000))
    hwe.pass <- row.names(hwe[which(hwe$Pr.exact < 0.05),])
    greater <- names(sum$Hexp)[sum$Hobs > sum$Hexp]
    remove2 <- hwe.pass[which(hwe.pass %in% greater)]
    print(paste0("Removed ", length(remove2), " additional loci with Ho >= ", het.cutoff, " and excess heterozygosity."))
    gi <- gi[loc=which(locNames(gi) %ni% remove2), drop=T]
  }
  
  # Coordinates file
  coords <- read.table("./data/coordinates.txt", header=T, check.names=F)
  coords <- coords[match(row.names(gi$tab), coords$Individual),]
  
  # Pops file for each zone, specifying which pops to keep
  meadows <- read.table("./data/meadows.txt", header=T, check.names=F)
  
  gi.edit = T # Indicate this section has been completed, gi has been processed
}



############################################################################
##########################     SPATIAL PCA     #############################
############################################################################

# Conduct sPCA
sp <- spca(gi, xy=coords[,c(3:4)], ask=F, type=5, scannf=F, d1=10, d2=20000, nfposi=3, nfnega=0)
pdf(paste0("./plots/sPCAplot.pdf"), width=10, height=10, useDingbats=F)
sPCAplot(sp, coords)
dev.off()
sPCAloci <- sp$c1
sPCAloci <- sPCAloci^2



############################################################################
##########################  CYCLE THRU ZONES   #############################
############################################################################

# Generate input files for NewHybrids based on top 1% of sPC for each zone
# Then run NewHybrids
burnin <- 10000
sweeps <- 20000
options(scipen = 999)
for(s in c(1:3)) {
  # Choose which ZONE
  if(s == 1) {
    # Meadows for P1
    P1pops <- c("4317","4370","4365",
                "4164","4391","3613") 
    # Meadows for P2
    P2pops <- c("2864","2836","2824",
                "2618","2594","2429",
                "2407","2384","2312",
                "2256") 
    zone <- "EN"
    p <- 0.05
  }
  if(s == 2) {
    # Meadows for P1
    P1pops <- c("359","377","387",
                "638","705","719",
                "758","1040","1369",
                "780") 
    # Meadows for P2
    P2pops <- c("2104","2429","1782",
                "2824","2836","2407",
                "2594","1951","1956",
                "1997","2021","2039",
                "2135","2256","2384",
                "2618") 
    zone <- "ES"
    p <- 0.02
  }
  if(s == 3) {
    # Meadows for P1
    P1pops <- c("1756","1779","1841",
                "2012","2030","2369",
                "2418","2421","2443",
                "2371","2385","2391",
                "2411","2073","2118",
                "2132") 
    # Meadows for P2
    P2pops <- c("2498","2147","2324",
                "2351","2410","2893",
                "2824","2594","2256",
                "2312","2384","2407",
                "2429","2618","2836",
                "2864","2977") 
    zone <- "EW"
    p <- 0.02
  }
  
  if( !file.exists("infiles") ) {
    dir.create("infiles")
  }
  setwd("infiles")
  if( !file.exists("structure") ) {
    dir.create("structure")
  }
  if( !file.exists("NewHybrids") ) {
    dir.create("NewHybrids")
  }
  if( !file.exists("HIest") ) {
    dir.create("HIest")
  }
  
  
  # Specify which pops to keep
  meadow <- meadows[meadows$Zone==s,1]
  
  # Use percent of loci retained in file names
  per <- p*100
  
  # Format files for output (using genind object)...
  
  # Select appropriate meadows for the zone
  gi.bgc <- gi[pop=which(popNames(gi) %in% meadow), drop=T]
  gi.bgc.p1 <- gi.bgc[pop=which(popNames(gi.bgc) %in% P1pops)]
  gi.bgc.p2 <- gi.bgc[pop=which(popNames(gi.bgc) %in% P2pops)]
  gi.temp <- gi.bgc
  
  # Drop locally (zone-wide) monomorphic loci
  gi.bgc <- gi.bgc[loc=which(gi.bgc@loc.n.all > 1), drop=T]
  keep <- locNames(gi.bgc)
  gi.bgc.p1 <- gi.bgc.p1[loc=which(locNames(gi.bgc.p1) %in% keep)]
  gi.bgc.p2 <- gi.bgc.p2[loc=which(locNames(gi.bgc.p2) %in% keep)]
  
  # Drop loci that are missing (> a.not.NA) from all pops
  keep <- colnames(gi.bgc@tab)[colSums(gi.bgc@tab/gi.bgc@tab,na.rm=T)/dim(gi.bgc@tab)[1] >= a.not.NA]
  keep <- unique(gsub("([IC]\\d+)\\.\\d+", "\\1", keep[!is.na(keep)], perl=T))
  gi.bgc <- gi.bgc[loc=which(locNames(gi.bgc) %in% keep), drop=T]
  gi.bgc.p1 <- gi.bgc.p1[loc=which(locNames(gi.bgc.p1) %in% keep)]
  gi.bgc.p2 <- gi.bgc.p2[loc=which(locNames(gi.bgc.p2) %in% keep)]
  
  # Drop loci that are too close (max.freq) to fixation
  gi.bgc.temp <- genind2genpop(gi.bgc, pop=factor(rep("All", dim(gi.bgc@tab)[1])), quiet=T)
  freqs <- tab(gi.bgc.temp, freq=T)
  freqs <- data.frame(freqs=as.numeric(freqs), locus=gi.bgc.temp@loc.fac)
  freqs <- aggregate(.~locus, freqs, FUN=max)
  remove <- as.character(freqs[which(freqs$freqs>max.freq),"locus"])
  gi.bgc <- gi.bgc[loc=which(locNames(gi.bgc) %ni% remove), drop=T]
  gi.bgc.p1 <- gi.bgc.p1[loc=which(locNames(gi.bgc.p1) %ni% remove)]
  gi.bgc.p2 <- gi.bgc.p2[loc=which(locNames(gi.bgc.p2) %ni% remove)]
  
  # Output STRUCTURE file
  gi.zone.structure <- gi.bgc
  gi.zone.structure <- genind2df(gi.zone.structure, oneColPerAll=T)
  ID <- data.frame(Ind=row.names(gi.zone.structure), pop=gi.zone.structure$pop, stringsAsFactors=F)
  ID$pop <- as.character(ID$pop)
  gi.zone.structure <- gi.zone.structure[,-1]
  gi.zone.structure.1 <- gi.zone.structure[,seq(1,length(gi.zone.structure),2)]
  gi.zone.structure.2 <- gi.zone.structure[,seq(2,length(gi.zone.structure),2)]
  gi.zone.structure <- data.frame(matrix(NA, nrow = nrow(gi.zone.structure)*2, 
                         ncol = ncol(gi.zone.structure)/2))
  gi.zone.structure[seq(1,nrow(gi.zone.structure),2),] <- gi.zone.structure.1
  gi.zone.structure[seq(2,nrow(gi.zone.structure),2),] <- gi.zone.structure.2
  names(gi.zone.structure) <- locNames(gi.bgc)
  ID.new <- data.frame(matrix(NA, nrow = nrow(ID)*2, ncol = 2))
  ID.new[seq(1,nrow(ID.new),2),] <- ID
  ID.new[seq(2,nrow(ID.new),2),] <- ID
  names(ID.new) <- names(ID)
  gi.zone.structure <- cbind(ID.new, gi.zone.structure)
  names(gi.zone.structure)[1:2] <- c("","")
  gi.zone.structure[gi.zone.structure=="NA"] <- "-9"
  write.table(gi.zone.structure, paste0("structure/", zone, ".stru"), 
              row.names=F, col.names=T, quote=F, sep="\t", na="-9")
  
  # Keep only top % of sPCA loci
  topSPC <- row.names(sPCAloci[rev(order(sPCAloci[,s])),])
  topSPC <- gsub("([IC]\\d+)\\.\\d+", "\\1", topSPC, perl=T)
  topSPC <- unique(topSPC)
  topSPC <- topSPC[1:round(length(topSPC)*p)]
  topSPC <- topSPC[which(topSPC %in% locNames(gi.bgc))]
  gi.bgc <- gi.bgc[loc=which(locNames(gi.bgc) %in% topSPC), drop=T]
  gi.bgc.p1 <- gi.bgc.p1[loc=which(locNames(gi.bgc.p1) %in% topSPC)]
  gi.bgc.p2 <- gi.bgc.p2[loc=which(locNames(gi.bgc.p2) %in% topSPC)]
  locOrder <- topSPC
  
  # Prepare NewHybrids files and run NewHybrids; assumes newhybs is in path (https://github.com/eriqande/newhybrids)
  gi.zone.NewHyb <- gi.bgc
  gi.zone.NewHyb <- genind2df(gi.zone.NewHyb)
  gi.zone.NewHyb <- gi.zone.NewHyb[c("pop", locOrder)]
  gi.zone.NewHyb[is.na(gi.zone.NewHyb)] <- "0000"
  zone.names <- data.frame(ind=row.names(gi.zone.NewHyb), pop=gi.zone.NewHyb[,1])
  write.table(zone.names, paste0("./NewHybrids/", zone, ".names.txt"), row.names=F, col.names=F, quote=F, sep="\t")
  gi.zone.NewHyb <- gi.zone.NewHyb[,-1]
  row.names(gi.zone.NewHyb) <- seq(1, nrow(gi.zone.NewHyb), 1)
  line1 <- paste("NumIndivs", nrow(gi.zone.NewHyb), sep=" ")
  line2 <- paste("NumLoci", ncol(gi.zone.NewHyb), sep=" ")
  line3 <- paste("Digits 2")
  line4 <- paste("Format Lumped")
  line5 <- paste("LocusNames", paste(locOrder, collapse=" "), sep=" ")
  header <- c(line1, line2, line3, line4, line5)
  header <- data.frame(header)
  write.table(header, paste0("./NewHybrids/", zone, ".txt"), row.names=F, col.names=F, quote=F)
  write.table(gi.zone.NewHyb, paste0("./NewHybrids/", zone, ".txt"), row.names=T, col.names=F, quote=F, sep="\t", append=T)
  command <- paste0("-d ", zone, ".txt --burn-in ", burnin, " --num-sweeps ", sweeps, " --no-gui")
  setwd("./NewHybrids")
  system2("newhybs", command)
  files.replace <- list.files()[grepl("^aa", list.files()) | grepl("^Echoed", list.files())]
  for(f in 1:length(files.replace)) {
    system2("mv", paste0(files.replace[f], " ", zone, "-", files.replace[f]))
  }
  setwd("../")
  
  # Prepare HIest files and run HIest
  gi.zone.HIest <- gi.bgc
  gi.zone.HIest <- genind2df(gi.zone.HIest, oneColPerAll=T)
  ID <- data.frame(Ind=row.names(gi.zone.HIest), pop=as.character(gi.zone.HIest$pop), stringsAsFactors=F)
  gi.zone.HIest <- gi.zone.HIest[,-1]
  gi.zone.HIest.1 <- gi.zone.HIest[,seq(1,length(gi.zone.HIest),2)]
  gi.zone.HIest.2 <- gi.zone.HIest[,seq(2,length(gi.zone.HIest),2)]
  G <- data.frame(matrix(NA, nrow = nrow(gi.zone.HIest)*2, 
                             ncol = ncol(gi.zone.HIest)/2))
  G[seq(1,nrow(G),2),] <- gi.zone.HIest.1
  G[seq(2,nrow(G),2),] <- gi.zone.HIest.2
  ID.new <- data.frame(matrix(NA, nrow = nrow(ID)*2, ncol = ncol(ID)))
  ID.new[seq(1,nrow(ID.new),2),] <- ID
  ID.new[seq(2,nrow(ID.new),2),] <- ID
  names(ID.new) <- c("Ind", "pop")
  names(G) <- locNames(gi.bgc)
  gi.bgc.p1@pop <- factor(rep("P1", length(gi.bgc.p1@pop)))
  gi.bgc.p2@pop <- factor(rep("P2", length(gi.bgc.p2@pop)))
  freq1 <- makefreq(genind2genpop(gi.bgc.p1, quiet=T), missing="mean", quiet=T)
  freq2 <- makefreq(genind2genpop(gi.bgc.p2, quiet=T), missing="mean", quiet=T)
  locus <- sub("([IC]\\d+)\\.\\d+", "\\1", names(freq1), perl=T)
  allele <- sub("[IC]\\d+\\.(\\d+)", "\\1", names(freq1), perl=T)
  P <- data.frame(Locus=locus, Allele=allele, P1=freq1, P2=freq2, stringsAsFactors=F)
  row.names(P) <- seq(1,nrow(P),1)
  names(G) <- seq(1:length(names(G)))
  P.loci <- data.frame(ID=names(G), Locus=unique(P$Locus))
  P.loci <- join(P.loci, P, by="Locus")
  P <- P.loci
  P <- P[,-2]
  names(P)[1] <- "Locus"
  P$Locus <- as.numeric(as.character(P$Locus))
  G[G=="NA"] <- NA
  H.est <- HIest(G, P, type="codominant", method="SANN", startgrid=100, iterations=1000, surf=T)
  H.class <- HIclass(G, P, type="codominant")
  H.test <- HItest(H.class, H.est)
  H.test <- cbind(ID, H.test)
  score <- data.frame(Ind=row.names(sp$li), score=sp$li[,s])
  H.test <- join(H.test, score, by="Ind")
  write.table(H.test, paste0("./HIest/", zone, "_HIest.txt"), row.names=F, col.names=T, quote=F, sep="\t")
  
  setwd("../")
}


# Plot barplots for HIest, NewHybrids, STRUCTURE, and PCA
zones <- c("EN","ES","EW")
for(i in 1:length(zones)) {
  results <- read.table(paste0("./infiles/NewHybrids/", zone, "-aa-PofZ.txt"), header=T)
  names(results) <- c("Ind", "Meadow", "P1", "P2", "F1", "F2", "P1-BC", "P2-BC")
  zone.names <- read.table(paste0("./infiles/NewHybrids/", zones[i], ".names.txt"), header=F)
  results[,1] <- zone.names[,1]
  results[,2] <- zone.names[,2]
  
  str <- read.table(paste0("./outfiles/structure/", zones[i], "_ind.txt"), header=T)
  str <- join(data.frame(Ind=results$Ind), str, by="Ind")
  
  if(zones[i]=="EN") {
    order <- data.frame(Meadow=
             c("4317","4370","4365","4164","4391","3613","4136","4146",
               "4324","3537","3452","3273","3384","3612","3615","3763",
               "4025","3400","3342","3414","3339","3272","3371","3424",
               "3420","3225","3200","2977","2893","2864","2836","2824",
               "2618","2594","2429","2407","2384","2312","2256"))
  }
  if(zones[i]=="ES") {
    order <- data.frame(Meadow=
             c("359","377","387","638","705","719","758","1040","1369",
               "780","1171","1070","733","1543","1569","942","1097",
               "1856","2977","2864","1815","2893","2312","1960","2026",
               "2059","2104","2429","1782","2824","2836","2407","2594",
               "1951","1956","1997","2021","2039","2135","2256","2384","2618"))
  }
  if(zones[i]=="EW") {
    order <- data.frame(Meadow=
              c("1756","1779","1841","2012","2030","2369","2418","2421",
                "2443","2371","2385","2391","2411","2073","2118","2132",
                "2498","2147","2324","2351","2410","2893","2824","2594",
                "2256","2312","2384","2407","2429","2618","2836","2864","2977"))
  }

  results <- join(order, results, by="Meadow")
  results[,c(1:2)] <- results[,c(2:1)]
  names(results)[1:2] <- c("Ind","Meadow")
  results$P1 <- results$P1 + results$F1
  results <- results[,-which(names(results) %in% c("F1"))]
  
  str <- join(order, str, by="Meadow")
  str[,c(1:2)] <- str[,c(2:1)]
  names(str)[1:2] <- c("Ind","Meadow")

  col6 = c("red","blue","purple","pink","cyan")
  classes <- c("P1","P2","F2","P1-BC","P2-BC")
  names(col6) = classes
  
  col2 = c("red","blue")
  admix <- c("P1","P2")
  names(col2) = admix
  
  if(results$P1[1] < 0.5) {
    names(results)[-c(1:2)] <- c("P2","P1","F2","P2-BC","P1-BC")
  }
  if(str$P1[1] < 0.5) {
    names(str)[-c(1:2)] <- c("P2","P1")
  }
  
  mdat = melt(results, id.vars=c("Ind", "Meadow"),
              variable.name="Hyb. Class", value.name="Probability")
  mdat$`Hyb. Class` = factor(mdat$`Hyb. Class`,levels=classes)
  mdat$`Hyb. Class` <- factor(mdat$`Hyb. Class`, levels=classes)
  mdat$Meadow <- factor(mdat$Meadow, levels=order$Meadow)
  
  mdat2 = melt(str, id.vars=c("Ind", "Meadow"),
              variable.name="Ancestry", value.name="Probability")
  mdat2$Ancestry = factor(mdat2$Ancestry,levels=admix)
  mdat2$Ancestry <- factor(mdat2$Ancestry, levels=admix)
  mdat2$Meadow <- factor(mdat2$Meadow, levels=order$Meadow)
  
  plot1 <- ggplot(mdat2, aes(x=Ind, y=Probability, fill=Ancestry, order=Ancestry)) +
    geom_bar(stat="identity", position="fill", width=1) +
    facet_grid(. ~ Meadow, drop=TRUE, space="free", scales="free") +
    theme(panel.grid=element_blank()) +
    theme(panel.background=element_rect(fill=NA, colour="grey25")) +
    theme(panel.spacing.x=grid:::unit(0.25, "lines")) +
    theme(strip.text.x = element_blank()) +
    theme(axis.title.x=element_blank()) +
    theme(axis.text.x=element_blank()) +
    theme(axis.ticks.x=element_blank()) +
    theme(strip.background=element_blank()) +
    theme(strip.text=element_text(size=12)) +
    theme(legend.key=element_rect(colour="grey25")) +
    scale_x_discrete(expand=c(0, 0)) +
    scale_y_continuous(expand=c(0, 0)) +
    scale_fill_manual(values=col2) +
    guides(fill=guide_legend(override.aes=list(colour=NULL)))
  
  plot2 <- ggplot(mdat, aes(x=Ind, y=Probability, fill=`Hyb. Class`, order=`Hyb. Class`)) +
    geom_bar(stat="identity", position="fill", width=1) +
    facet_grid(. ~ Meadow, drop=TRUE, space="free", scales="free") +
    theme(panel.grid=element_blank()) +
    theme(panel.background=element_rect(fill=NA, colour="grey25")) +
    theme(panel.spacing.x=grid:::unit(0.25, "lines")) +
    theme(strip.text.x = element_blank()) +
    theme(axis.title.x=element_blank()) +
    theme(axis.text.x=element_blank()) +
    theme(axis.ticks.x=element_blank()) +
    theme(strip.background=element_blank()) +
    theme(strip.text=element_text(size=12)) +
    theme(legend.key=element_rect(colour="grey25")) +
    scale_x_discrete(expand=c(0, 0)) +
    scale_y_continuous(expand=c(0, 0)) +
    scale_fill_manual(values=col6) +
    guides(fill=guide_legend(override.aes=list(colour=NULL)))
  
  # Plot HIest
  H.test <- read.table(paste0("./infiles/HIest/", zones[i], "_HIest.txt"), header=T, sep="\t")
  H.test$S <- 1 - H.test$S
  tern.data <- H.test[c("S","H")]
  tern.data$H <- tern.data$H * sin(45)
  crd = coord_tern()
  tern.data <- plyr::rename(tern.data,c("S"="x","H"="y"))
  tern.data <- tlr2xy(tern.data,crd,inverse = T)
  advanced <- which(H.test$Best.class!="class100" & H.test$Best.class!="class001" & H.test$c2==F)
  tern.adv.data <- tern.data[advanced,]
  plot3 <- ggtern(data=tern.data,aes(x,y,z)) + 
    geom_mask() + theme_bw() +
    geom_point(cex=2.5, aes(color=factor(ID$Best.class, 
                                levels=c("P1","P2","F2","P1-BC","P2-BC")))) + 
    scale_color_manual(values=c("red","blue","purple","pink","cyan"),
                       labels=c("P1","P2","F2","P1-BC","P2-BC")) + 
    geom_point(data=tern.adv.data, pch=1, cex=3) +
    geom_point(data=tern.adv.data, pch=1, cex=4.5) +
    scale_T_continuous(breaks = seq(0,1,0.2), labels = c(0,0.2,0.4,0.6,0.8,1.0)) +
    scale_L_continuous(breaks = NULL, labels = NULL) +
    scale_R_continuous(breaks = seq(0,1,0.2), labels = c(0,0.2,0.4,0.6,0.8,1.0)) +
    labs(x="",y="",z="",color='Hyb. Class') +
    Rarrowlab("Hybrid Index") +
    Tarrowlab("Inter-Lineage Heterozygosity") +
    theme_showarrows() +
    theme(legend.position="none",
          plot.margin=unit(c(0,0,0,0), "null")) +
    theme(tern.axis.ticks.length.major=unit(6.0,'mm'),
          tern.axis.ticks.length.minor=unit(0,'mm')) +
    theme(axis.text=element_text(size=12,face="bold",color="black"),
          axis.title=element_text(size=14,face="bold",color="black"))

  # Plot PCA
  dim.stru <- read.table(paste0("./infiles/structure/", zones[i], ".stru"), header=T, sep="\t", na="-9")
  gi.zone.structure <- read.structure(paste0("./infiles/structure/", zones[i], ".stru"),
                       n.ind = dim(dim.stru)[1]/2, n.loc = dim(dim.stru)[2]-2, 
                       onerowperind = F, col.lab = 1, col.pop = 2, row.marknames = 1,
                       NA.char = "-9", ask = F, quiet=T)
  pca.data <- gi.zone.structure@tab
  for(j in 1:ncol(pca.data)){pca.data[is.na(pca.data[,j]), j] <- mean(pca.data[,j], na.rm = TRUE)}
  pca <- prcomp(pca.data, center=T, scale.=T)
  var1 <- round(summary(pca)[6][[1]][2,1]*100, 2)
  var2 <- round(summary(pca)[6][[1]][2,2]*100, 2)
  ID <- join(data.frame(Ind=row.names(pca.data)), H.test, by="Ind")
  ID$Best.class <- as.character(ID$Best.class)
  ID$Best.class[which(ID$Best.class=="class001")] <- "P1"
  ID$Best.class[which(ID$Best.class=="class100")] <- "P2"
  ID$Best.class[which(ID$Best.class=="class010")] <- "F1"
  ID$Best.class[which(ID$Best.class=="class121")] <- "F2"
  ID$Best.class[which(ID$Best.class=="class011")] <- "P1-BC"
  ID$Best.class[which(ID$Best.class=="class110")] <- "P2-BC"
  col.class <- data.frame(Best.class=classes, col6=col6)
  ID <- join(ID, col.class, by="Best.class")
  
  pca.plot.data <- pca$x[,c(1:2)]
  if(i==1) {pca.plot.data[,1] <- pca.plot.data[,1] * -1}
  if(i==3) {pca.plot.data[,2] <- pca.plot.data[,2] * -1}
  pca.adv.data <- pca.plot.data[advanced,]
  plot4 <- ggplot(pca.plot.data, aes(PC1,PC2)) +
    geom_point(cex=2.5, aes(color=factor(ID$Best.class, 
               levels=c("P1","P2","F2","P1-BC","P2-BC")))) + 
    scale_color_manual(values=c("red","blue","purple","pink","cyan"),
                       labels=c("P1","P2","F2","P1-BC","P2-BC")) + 
    theme_bw() + theme(legend.position="none") +
    geom_point(data=pca.adv.data, pch=1, cex=3, aes(x=PC1,y=PC2)) +
    geom_point(data=pca.adv.data, pch=1, cex=4.5, aes(x=PC1,y=PC2)) +
    xlab(paste0("PC1 (", var1, "%)")) + ylab(paste0("PC2 (", var2, "%)"))
  
  lay <- rbind(c(1,1,1,1),
               c(1,1,1,1),
               c(2,2,3,3),
               c(2,2,3,3))
  
  g2 <- ggplotGrob(plot1)
  g3 <- ggplotGrob(plot2)
  g4 <- ggplotGrob(plot4)
  g5 <- ggplotGrob(plot3)
  g <- rbind(g2, g3, size = "first")
  g$widths <- unit.pmax(g2$widths, g3$widths)
  g <- rbind(g2, g3, size = "first")
  g$widths <- unit.pmax(g2$widths, g3$widths)
  
  if( !file.exists("plots") ) {
    dir.create("plots")
  }
  pdf(paste0("./plots/", zones[i], "barplot.pdf"), width=10, height=10, useDingbats=F)
  print({ grid.arrange(grobs = list(g,g4,g5),layout_matrix = lay) })
  dev.off()
}






############################################################################
##########################     SIMULATIONS     #############################
############################################################################


# Individuals x and y are represented by vectors of length L (L markers), 
# where each entry is 0, 1, or 2 (homozygous absence of allele 1, 
# heterozygous, homozygous presence of allele 1)
sex.fn <- function(x,y){
  L <- length(x)
  rbinom(L,1,x/2) + rbinom(L,1,y/2)
}

# Use the closed form MLE for diagnostic markers, for simplicity
drift.fn <- function(Genotypes,Generations,imm=0){
  L <- ncol(Genotypes)
  N <- nrow(Genotypes)
  for(j in 1:Generations){
    Fn <- Genotypes
    Fn[1,] <- sex.fn(Genotypes[sample(1:N,1),],Genotypes[sample(1:N,1),])
    for(i in 2:N){Fn[i,] <- sex.fn(Genotypes[sample(1:N,1),],Genotypes[sample(1:N,1),])}
    Nm <- rbinom(1,N,imm)
    N1 <- rbinom(1,Nm,1/2)
    N2 <- Nm-N1
    Genotypes <- rbind(Fn[1:(N-Nm),],matrix(0,nrow=N1,ncol=L),matrix(2,nrow=N2,ncol=L))
  }
  S <- rowMeans(Fn)/2 # MLE ancestry for diagnostic markers
  H <- rowSums(Fn==1)/L # MLE heterozygosity for diagnostic markers
  list(Genotypes=Fn,AH=data.frame(S,H))
}

## Make parentals
makeParentals <- function(L,N,m) {
  x <- rep(1,L) # allele frequencies in parental population 1
  y <- rep(0,L) # allele frequencies in parental population 2
  P1 <- P2 <- matrix(NA,ncol=L,nrow=N)
  for(i in 1:N){
    P1[i,] <- sex.fn(2*x,2*x)
    P2[i,] <- sex.fn(2*y,2*y)
  }
  n1 <- round(m*N,0)
  n2 <- round((1-m)*N,0)	
  Pb <- rbind(P1[1:n1,],P2[1:n2,]) # mixture of pure parentals
  return(Pb)
}

L <- 25 # number of markers
N <- 50 # number of individuals in population
m <- 0.1 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1a.out <- drift.fn(Pb,1,imm=0)
F2a.out <- drift.fn(F1a.out$G,1,imm=0)
F5a.out <- drift.fn(F2a.out$G,3,imm=0)
F50a.out <- drift.fn(F5a.out$G,45,imm=0)
F100a.out <- drift.fn(F50a.out$G,50,imm=0)
F500a.out <- drift.fn(F100a.out$G,400,imm=0)

m <- 0.2 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1b.out <- drift.fn(Pb,1,imm=0)
F2b.out <- drift.fn(F1b.out$G,1,imm=0)
F5b.out <- drift.fn(F2b.out$G,3,imm=0)
F50b.out <- drift.fn(F5b.out$G,45,imm=0)
F100b.out <- drift.fn(F50b.out$G,50,imm=0)
F500b.out <- drift.fn(F100b.out$G,400,imm=0)

m <- 0.3 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1c.out <- drift.fn(Pb,1,imm=0)
F2c.out <- drift.fn(F1c.out$G,1,imm=0)
F5c.out <- drift.fn(F2c.out$G,3,imm=0)
F50c.out <- drift.fn(F5c.out$G,45,imm=0)
F100c.out <- drift.fn(F50c.out$G,50,imm=0)
F500c.out <- drift.fn(F100c.out$G,400,imm=0)

m <- 0.4 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1d.out <- drift.fn(Pb,1,imm=0)
F2d.out <- drift.fn(F1d.out$G,1,imm=0)
F5d.out <- drift.fn(F2d.out$G,3,imm=0)
F50d.out <- drift.fn(F5d.out$G,45,imm=0)
F100d.out <- drift.fn(F50d.out$G,50,imm=0)
F500d.out <- drift.fn(F100d.out$G,400,imm=0)

m <- 0.5 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1e.out <- drift.fn(Pb,1,imm=0)
F2e.out <- drift.fn(F1e.out$G,1,imm=0)
F5e.out <- drift.fn(F2e.out$G,3,imm=0)
F50e.out <- drift.fn(F5e.out$G,45,imm=0)
F100e.out <- drift.fn(F50e.out$G,50,imm=0)
F500e.out <- drift.fn(F100e.out$G,400,imm=0)

m <- 0.6 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1f.out <- drift.fn(Pb,1,imm=0)
F2f.out <- drift.fn(F1f.out$G,1,imm=0)
F5f.out <- drift.fn(F2f.out$G,3,imm=0)
F50f.out <- drift.fn(F5f.out$G,45,imm=0)
F100f.out <- drift.fn(F50f.out$G,50,imm=0)
F500f.out <- drift.fn(F100f.out$G,400,imm=0)

m <- 0.7 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1g.out <- drift.fn(Pb,1,imm=0)
F2g.out <- drift.fn(F1g.out$G,1,imm=0)
F5g.out <- drift.fn(F2g.out$G,3,imm=0)
F50g.out <- drift.fn(F5g.out$G,45,imm=0)
F100g.out <- drift.fn(F50g.out$G,50,imm=0)
F500g.out <- drift.fn(F100g.out$G,400,imm=0)

m <- 0.8 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1h.out <- drift.fn(Pb,1,imm=0)
F2h.out <- drift.fn(F1h.out$G,1,imm=0)
F5h.out <- drift.fn(F2h.out$G,3,imm=0)
F50h.out <- drift.fn(F5h.out$G,45,imm=0)
F100h.out <- drift.fn(F50h.out$G,50,imm=0)
F500h.out <- drift.fn(F100h.out$G,400,imm=0)

m <- 0.9 # initial admixture proportion
Pb <- makeParentals(L,N,m)
F1i.out <- drift.fn(Pb,1,imm=0)
F2i.out <- drift.fn(F1i.out$G,1,imm=0)
F5i.out <- drift.fn(F2i.out$G,3,imm=0)
F50i.out <- drift.fn(F5i.out$G,45,imm=0)
F100i.out <- drift.fn(F50i.out$G,50,imm=0)
F500i.out <- drift.fn(F100i.out$G,400,imm=0)


# Plot all simulated data!
par(mfrow=c(1,3),mar=c(3.5,3.5,.5,.5),mex=1,cex=1.2,mgp=c(2,1,0),lwd=2)
plot(c(0,.5,1,0),c(0,1,0,0),type="l",xlab=expression(italic(S)),
     ylab=expression(italic(H[I])),lwd=2,cex.lab=1.5,cex.axis=1.5,bty="n",
     xaxt='n',yaxt='n')
points(F2a.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2b.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2c.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2d.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2e.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2f.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2g.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2h.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F2i.out$A, pch=19, col=rgb(0.5,0.5,0.5,0.5))
points(F1a.out$A, pch=19, col="black")
points(F1b.out$A, pch=19, col="black")
points(1,0, pch=21, col="black", bg="white")
points(0,0, pch=21, col="black", bg="white")
legend(-0.05, 1.15, c("P","F1","F2"),
       pch = c(21,19,19), bty = "n",
       col = c("black","black",rgb(0.25,0.25,0.25,0.5)),
       cex = 0.9, pt.cex=1.0, y.intersp=0.25, x.intersp=0.5, text.font=2)
axis(1,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=-0.75)
axis(2,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=0.75)

plot(c(0,.5,1,0),c(0,1,0,0),type="l",xlab=expression(italic(S)),
     ylab=expression(italic(H[I])),lwd=2,cex.lab=1.5,cex.axis=1.5,bty="n",
     xaxt='n',yaxt='n')
points(F5a.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5b.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5c.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5d.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5e.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5f.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5g.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5h.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F5i.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F50a.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50b.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50c.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50d.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50e.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50f.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50g.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50h.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F50i.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
legend(-0.05, 1.15, c("F5","F50"),
       pch = c(19,19), bty = "n",
       col = c(rgb(0.1,0.1,0.1),rgb(0.5,0.5,0.5,0.5)),
       cex = 0.9, pt.cex=1.0, y.intersp=0.25, x.intersp=0.5, text.font=2)
axis(1,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=-0.75)
axis(2,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=0.75)

plot(c(0,.5,1,0),c(0,1,0,0),type="l",xlab=expression(italic(S)),
     ylab=expression(italic(H[I])),lwd=2,cex.lab=1.5,cex.axis=1.5,bty="n",
     xaxt='n',yaxt='n')
points(F100a.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100b.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100c.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100d.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100e.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100f.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100g.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100h.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F100i.out$A, pch=19, col=rgb(0.1,0.1,0.1,0.8))
points(F500a.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500b.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500c.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500d.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500e.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500f.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500g.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500h.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
points(F500i.out$A, pch=19, col=rgb(0.7,0.7,0.7,0.5))
legend(-0.05, 1.15, c("F100","F500"),
       pch = c(19,19), bty = "n",
       col = c(rgb(0.1,0.1,0.1),rgb(0.5,0.5,0.5,0.5)),
       cex = 0.9, pt.cex=1.0, y.intersp=0.25, x.intersp=0.5, text.font=2)
axis(1,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=-0.75)
axis(2,at=c(0,0.5,1),labels=c(0,0.5,1),lwd=2,padj=0.75)
